//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "Object.hpp"
//------------------------------------------------------------------------------
#include "Group.hpp"
#include "Screen.hpp"
//------------------------------------------------------------------------------
#include "MyException.hpp"
//------------------------------------------------------------------------------
#include <cmath>
#include <string>
#include <iostream>
//------------------------------------------------------------------------------
#include "AllObj.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
/*#if DEBUG == 1
    int g_debugObjCount = 0;
#endif*/
//------------------------------------------------------------------------------
using namespace std;
//------------------------------------------------------------------------------
Object::Object(Group *parent, const std::string &name) : m_parent(parent), m_name(name)
{
    m_pos = Vector2D(0, 0);
    m_speed = Vector2D(0, 0);
    m_angularVelocity = 0.f;
    m_rotation = 0;

    /*#if DEBUG == 1
        g_debugObjCount++;
    #endif*/
}
//------------------------------------------------------------------------------
/*#if DEBUG == 1
Object::Object(const Object &obj)
{
    m_parent = obj.m_parent;
    m_name = obj.m_name;

    m_userNb = obj.m_userNb;
    m_userString = obj.m_userString;

    // Only accessible via Get* /Set*
    m_speed = obj.m_speed;
    m_angularVelocity = obj.m_angularVelocity;

    m_pos = obj.m_pos;
    m_rotation = obj.m_rotation;

    g_debugObjCount++;
}
#endif*/
//------------------------------------------------------------------------------
Object::~Object()
{
    /*#if DEBUG == 1
        cout << GetName() << " destroyed" << endl;
        g_debugObjCount--;
    #endif*/
}
//------------------------------------------------------------------------------
/*#if DEBUG == 1
int Object::GetDebugCount()
{
    return g_debugObjCount;
}
#endif*/
//------------------------------------------------------------------------------
Group *Object::GetParent()
{
    return m_parent;
}
//------------------------------------------------------------------------------
void Object::OnChangeScreen(Screen *backScreen, Screen *newScreen)
{

}
//------------------------------------------------------------------------------
void Object::SetParent(Group *parent)
{
    m_parent = parent;
}
//------------------------------------------------------------------------------
/** Set user data */
void Object::SetUserData(const std::string &id, const luabind::object &data)
{
    m_userData[id] = data;
}
//------------------------------------------------------------------------------
/** Get user data */
luabind::object Object::GetUserData(const std::string &id)
{
    std::map<std::string, luabind::object>::iterator it = m_userData.find(id);

    if(it != m_userData.end())
    {
        return it->second;
    }
    else
    {
        MY_THROW("Object::GetUserData : Key '" << id << "' not found in object '" << GetName() << "' !");
    }
}
//------------------------------------------------------------------------------
/** Return true if the user data exists, false otherwise */
bool Object::UserDataExists(const std::string &id)
{
    return(m_userData.find(id) != m_userData.end());
}
//------------------------------------------------------------------------------
Screen *Object::GetParentScreen()
{
    if(GetParent() == NULL)
    {
        if(GetType() == OBJ_TYPE_DRAWABLE_SCREEN)
        {
            return static_cast<Screen*>(this);
        }
        else
        {
            MY_THROW("Main parent of '" << GetName() << "' isn't a Screen");
        }
    }
    if(GetParent()->GetType() == OBJ_TYPE_DRAWABLE_SCREEN)
    {
        return static_cast<Screen*>(GetParent());
    }
    else
    {
        return(GetParent()->GetParentScreen());
    }
}

//------------------------------------------------------------------------------
std::string Object::GetName() const
{
    return m_name;
}
//------------------------------------------------------------------------------
/*void Object::RegisterClassLua(lua_State *myLuaState)
{
    luabind::module(myLuaState)
    [
        luabind::class_<Object>("Object")
            //.def(luabind::constructor<Group*, const std::string&>())
            // Position
            .def("GetPosition", (Vector2D(Object::*)(void) const) &Object::GetPosition)
            .def("SetPosition", (void(Object::*)(const Vector2D&)) &Object::SetPosition)
            .def("GetRotation", &Object::GetRotation)
            .def("SetRotation", &Object::SetRotation)

            .def("GetPositionX", &Object::GetPositionX)
            .def("GetPositionY", &Object::GetPositionY)
            .def("SetPositionX", &Object::SetPositionX)
            .def("SetPositionY", &Object::SetPositionY)

            .def("GetGlobalPosition", &Object::GetGlobalPosition)
            .def("GetGlobalRotation", &Object::GetGlobalRotation)

            // Move
            .def("Rotate", &Object::Rotate)
            .def("Move", &Object::Move)

            // Name
            .def("GetName", &Object::GetName)
            .def("SetName", &Object::SetName)

            // Speed
            .def("SetSpeed", &Object::SetSpeed)
            .def("GetSpeed", &Object::GetSpeed)
            .def("SetAngularVelocity", &Object::SetAngularVelocity)
            .def("GetAngularVelocity", &Object::GetAngularVelocity)

            .def("GetType", &Object::GetType)

            // Parent
            .def("ChangeParent", &Object::ChangeParent)
            .def("Destroy", &Object::Destroy)
            .def("GetParent", &Object::GetParent)

            // Clone & Create
            .def("Clone", &Object::Clone)
    ];
}*/
//------------------------------------------------------------------------------
void Object::GetPolarPosition(float &dist, float &angle) const
{
    LinePtPt line(Vector2D(0, 0), GetPosition());
    dist = line.GetSize();
    if(dist == 0)
    {
        angle = 0;
        return;
    }

    Vector2D vect = line.pt2 - line.pt1;
    angle = - 2 * atan(vect.y / (vect.x + dist));

    return;
}
//------------------------------------------------------------------------------
void Object::SetPolarPosition(float dist, float angle)
{
    if(dist != 0)
    {
        SetPosition(Vector2D(dist * cos(angle), - dist * sin(angle)));
    }
    else
    {
        SetPosition(Vector2D(0, 0));
    }
}
//------------------------------------------------------------------------------
void Object::Destroy()
{
    if(m_parent != NULL)
    {
        try
        {
            m_parent->AddToDelete(this);
        }
        catch(std::exception &e)
        {
            cerr << e.what() << endl;
        }
    }
    else
    {
        MY_THROW("Error: No parent !!")
        /*ChangeParent(NULL);
        delete this;*/
    }
}
//------------------------------------------------------------------------------
void Object::SetName(const std::string &str)
{
    m_name = str;
}
//------------------------------------------------------------------------------
Vector2D Object::GetGlobalPosition() const
{
    if(m_parent != NULL)
    {
        return m_parent->GetGlobalPosition() + m_pos;
    }
    else
    {
        return m_pos;
    }
}
//------------------------------------------------------------------------------
void Object::SetGlobalPosition(const Vector2D &pos)
{
    if(m_parent != NULL)
    {
        SetPosition(pos - m_parent->GetGlobalPosition());
    }
    else
    {
        SetPosition(pos);
    }
}
//------------------------------------------------------------------------------
float Object::GetGlobalRotation() const
{
    if(m_parent != NULL)
    {
        return m_parent->GetGlobalRotation() + m_rotation;
    }
    else
    {
        return m_rotation;
    }
}
//------------------------------------------------------------------------------
void Object::SetGlobalRotation(float rotation)
{
    if(m_parent != NULL)
    {
        SetRotation(rotation - m_parent->GetGlobalRotation());
    }
    else
    {
        SetRotation(rotation);
    }
}
//------------------------------------------------------------------------------
Vector2D Object::GetPosition() const
{
    return m_pos;
}
//------------------------------------------------------------------------------
void Object::SetPosition(const Vector2D &pos)
{
    m_pos = pos;
}
//------------------------------------------------------------------------------
float Object::GetRotation() const
{
    return m_rotation;
}
//------------------------------------------------------------------------------
void Object::SetRotation(float rotation)
{
    m_rotation = floatingModulo(rotation, float(2.f * M_PI));
}
//------------------------------------------------------------------------------
void Object::Rotate(float angle)
{
    if(angle != 0)
    {
        SetRotation(GetRotation() + angle);
    }
}
//------------------------------------------------------------------------------
void Object::SetSpeed(const Vector2D &speed)
{
    m_speed = speed;
}
//------------------------------------------------------------------------------
Vector2D Object::GetSpeed() const
{
    return m_speed;
}
//------------------------------------------------------------------------------
void Object::SetAngularVelocity(float speed)
{
    m_angularVelocity = speed;
}
//------------------------------------------------------------------------------
float Object::GetAngularVelocity() const
{
    return m_angularVelocity;
}
//------------------------------------------------------------------------------
void Object::Move(const Vector2D &mvt)
{
    if(mvt != Vector2D(0, 0))
    {
        SetPosition(GetPosition() + mvt);
    }
}
//------------------------------------------------------------------------------
void Object::ProceedMvt(float time)
{
    Move(GetSpeed() * time);
    Rotate(GetAngularVelocity() * time);
}
//------------------------------------------------------------------------------
Vector2D Object::PointRotate(const Vector2D &centre, const Vector2D &pt, float angle) const
{
    Vector2D newPt;
    newPt.x = ((pt.x - centre.x) * cos(angle)) - ((centre.y - pt.y) * sin(angle));
    newPt.y = ((pt.y - centre.y) * cos(angle)) - ((pt.x - centre.x) * sin(angle));
    return newPt;
}
//------------------------------------------------------------------------------
long Object::GetUniqueId() const
{
    return reinterpret_cast<long>(this);
}
//------------------------------------------------------------------------------
// Cast
#define X(typeName, typeId) \
        typeName *Object::CastInto ## typeName() \
        { \
            if(GetType() == typeId) \
            { \
                return static_cast<typeName*>(this); \
            } \
            else \
            { \
                MY_THROW("Object::CastInto "# typeName "() : can't cast, bad type !"); \
            } \
        }
    X_OBJ_LIST
#undef X
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
} // namespace En
