//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "PhysicWorld.hpp"
//------------------------------------------------------------------------------
#include "MyException.hpp"

#include <iostream>
#include <utility>
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
using namespace std;
//------------------------------------------------------------------------------
PhysicWorld::PhysicWorld()
{
    //ctor
}
//------------------------------------------------------------------------------
PhysicWorld::~PhysicWorld()
{
    if(!m_phTbl.empty())
    {
        cerr << "Warning: PhysicWorld not empty when deleted !!" << endl;
        cerr << "Left: " << m_phTbl.size() << endl;
    }
}
//------------------------------------------------------------------------------
void PhysicWorld::Attach(PhysicObject *obj)
{
    // FIXME: delete test
    vector<PhysicObject*> &tbl = m_phTbl[obj->GetPhysicGroup()];

    if(find(tbl.begin(), tbl.end(), obj) != tbl.end())
    {
        MY_THROW("Object already exists ; Name : '" << obj->GetName() << "'");
    }
    tbl.push_back(obj);
}
//------------------------------------------------------------------------------
void PhysicWorld::Detach(PhysicObject *obj)
{
    // FIXME: Accelerer la recherche avec des map de pointeur !
    std::map<int, std::vector<PhysicObject*> >::iterator mapIt = m_phTbl.find(obj->GetPhysicGroup());
    vector<PhysicObject*> &tbl = mapIt->second;

    for(std::vector<PhysicObject*>::iterator it = tbl.begin(); it != tbl.end(); it++ )
    {
        if(obj == *it)
        {
            it = tbl.erase(it) - 1;

            if(tbl.empty())
            {
                m_phTbl.erase(mapIt);
            }

            return;
        }
    }

    MY_THROW("PhysicWorld::Detach() : Object '" << obj->GetName() << "' doesn't exist !");
}
//------------------------------------------------------------------------------
void PhysicWorld::ProceedIntersection()
{
    for(std::vector<std::pair<int, int> > ::iterator it = m_phGroupTbl.begin(); it != m_phGroupTbl.end(); it++ )
    {
        ProceedIntersection(it->first, it->second);
    }
}
//------------------------------------------------------------------------------
void PhysicWorld::AddPhysicGroup(int phGroupId1, int phGroupId2)
{
    for(std::vector<std::pair<int, int> > ::iterator it = m_phGroupTbl.begin(); it != m_phGroupTbl.end(); it++ )
    {
        if((it->first == phGroupId1 && it->second == phGroupId2) || (it->first == phGroupId2 && it->second == phGroupId1))
        {
            MY_THROW("PhysicWorld::AddPhysicGroup() : This group pair alredy exist !");
        }
    }

    m_phGroupTbl.push_back(std::pair<int, int>(phGroupId1, phGroupId2));
}
//------------------------------------------------------------------------------
void PhysicWorld::ProceedIntersection(int phGroupId1, int phGroupId2)
{
    std::map<int, std::vector<PhysicObject*> >::iterator itGrp1 = m_phTbl.find(phGroupId1);
    if(itGrp1 == m_phTbl.end())
    {
        return; // Nothing to intersect
    }

    std::map<int, std::vector<PhysicObject*> >::iterator itGrp2 = m_phTbl.find(phGroupId2);
    if(itGrp2 == m_phTbl.end())
    {
        return; // Nothing to intersect
    }

    std::vector<PhysicObject*> &tbl1 = itGrp1->second;
    std::vector<PhysicObject*> &tbl2 = itGrp2->second;

    Vector2D pos(0, 0);


    if(itGrp1 == itGrp2) // Inter group => less test to do
    {
        for(size_t i = 0; i < tbl1.size(); i++)
        {
            for(size_t j = i + 1; j < tbl2.size(); j++)
            {
                if(tbl1[i]->Intersect(*tbl2[j], pos))
                {
                    tbl1[i]->OnIntersect(tbl1[i], tbl2[j], pos);
                    tbl2[j]->OnIntersect(tbl1[i], tbl2[j], pos);
                }
            }
        }
    }
    else
    {
        for(size_t i = 0; i < tbl1.size(); i++)
        {
            for(size_t j = 0; j < tbl2.size(); j++)
            {
                if(tbl2[j]->Intersect(*tbl1[i], pos))
                {
                    tbl1[i]->OnIntersect(tbl1[i], tbl2[j], pos);
                    tbl2[j]->OnIntersect(tbl1[i], tbl2[j], pos);
                }
            }
        }
    }
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
} // namespace En
