-- Create screen
gameScreen = CreateScreen("gameScreen", ScreenConfig(screenSize))
gameScreen:ShowCursor(false)

------------------------------------------------------------
-- Enable / Diable weapon
------------------------------------------------------------
function enableWeapon(name, isEnabled)
	if name == "laser" then
		canUseLaser = isEnabled
		previousLaserTime = laserWaitTime
		laserAmmoCircle:Show(isEnabled)
	elseif name == "freezeWeapon" then
		canUseFreezeWeapon = isEnabled
		previousFreezeWeaponTime = laserWaitTime
		freezeWeaponAmmoCircle:Show(isEnabled)
	elseif name == "sight" then
		gunGroup:FindByName("sight"):Show(isEnabled)
		gunGroup:FindByName("sightGen"):Show(isEnabled)
	else
		printError("Unknown weapon '" .. name .. "' !")
	end
end

------------------------------------------------------------
-- Destroying weapon (used to explode enemies with no bullet)
------------------------------------------------------------
destroyingWeapon = gameScreen:CreateGroup("destroyingWeapon")
destroyingWeapon:Show(false)
destroyingWeapon:SetUserData("damage", 3000)

local tmp = destroyingWeapon:CreatePhCircle(phGroupBullets, "phLaser", Vector2D(0, 0), 10)

------------------------------------------------------------
-- Enemy group => contains all enemy to count them
------------------------------------------------------------
enemyContainer = gameScreen:CreateGroup("enemyContainer")

-- Destroy enemies with a visual explosion
function BurstAllEnemies()
	local objTbl = enemyContainer:GetChildTable()
	for i = 1, #objTbl do
		local tmp =  objTbl[i]
		
		if tmp:UserDataExists("points") then -- Enemy
			local phEnemy = nil
			if tmp:GetUserData("enemyType")  == "bonus" then
				phEnemy = tmp:FindByName("phBidon")
			else
				phEnemy = tmp:FindByName("ph")
			end
			tmp:GetUserData("OnHitFn")(destroyingWeapon:FindByName("phLaser"), phEnemy, tmp:GetPosition())			
		end
	end
end

function DestroyAllEnemies()
	local tbl = enemyContainer:GetChildTable()
	
	for i = 1, #tbl do
		local parent = tbl[i]
	
		if parent:UserDataExists("eventToDestroy") then -- We destroy the event
			parent:GetUserData("eventToDestroy"):Destroy()
		end
	
		parent:Destroy() -- We destroy the group
	end
end

------------------------------------------------------------
-- Events
------------------------------------------------------------

-- Margin, to destroy objects that are outside the screen
margin = 300
tmp = gameScreen:CreatePhRect(phGroupMargin, "outline", Vector2D(screenSize.x + margin * 2, screenSize.y + margin * 2))
tmp:SetPosition(Vector2D(-margin, -margin))
tmp:SetEventFunction(gameOnObjectOutScreen)

gameScreen:AddPhysicGroup(phGroupMargin, phGroupEnemies)
gameScreen:AddPhysicGroup(phGroupMargin, phGroupBullets)

-- Bullet intersection
gameScreen:AddPhysicGroup(phGroupBullets, phGroupEnemies)

-- Shield fluctuation
shieldFluctuationChrono = gameScreen:RegisterChronometer(gameOnShieldFluctuation, 1.5, false)

-- Shield Shock
gameScreen:AddPhysicGroup(phGroupShield, phGroupEnemies)

-- Mouse move event
gameScreen:SetMouseRelative(true)
gameScreen:RegisterMouseMoveEvent(gameOnMouseMove)

-- On frame
previousShootTime = shootWaitTime
previousLaserTime = laserWaitTime
previousFreezeWeaponTime = freezeWeaponWaitTime

gameScreen:RegisterFrameEvent(gameOnFrame)

-- Enemy timer
spawnTimer = gameScreen:RegisterTimer(gameOnTimer, 0.3, true)

-- Event Store
gameScreen:RegisterButtonEvent(gameOnStore, "Tab", true)

-- Event quit
gameScreen:RegisterButtonEvent(gameOnQuit, "Escape", true)

-- Laser event
gameScreen:RegisterButtonEvent(gameOnLaserBt, "MouseRight", true)

-- Freeze weapon event
gameScreen:RegisterButtonEvent(gameOnFreezeBt, "Space", true)

-- Clouds mvt
gameScreen:RegisterFrameEvent(gameCloudsMvt)


------------------------------------------------------------
-- Design
------------------------------------------------------------
-- Design
local tmp = gameScreen:CreateSprite("01_background")
tmp:SetZpos(-10000)
tmp:SetImage(tex01background)

local tmp = gameScreen:CreateSprite("02_clouds_p1")
tmp:SetZpos(-990)
tmp:SetImage(tex02cloudsP1)
tmp.position = Vector2D(0, 110)
cloudsSprP1 = tmp

local tmp = gameScreen:CreateSprite("02_clouds_p2")
tmp:SetZpos(-990)
tmp:SetImage(tex02cloudsP2)
tmp.position = Vector2D(-tmp:GetSize().x, 110)
cloudsSprP2 = tmp

local tmp = gameScreen:CreateSprite("03_city")
tmp:SetZpos(-980)
tmp:SetImage(tex03city)
tmp.position = Vector2D(0, 526)
gameDesignCity = tmp

local tmp = gameScreen:CreateSprite("04_ground")
tmp:SetZpos(-970)
tmp:SetImage(tex04ground)
tmp.position = Vector2D(0, 805)
gameDesignGround = tmp

local tmp = gameScreen:CreateSprite("05_fence")
tmp:SetZpos(-960)
tmp:SetImage(tex05fence)
tmp.position = Vector2D(0, 510)
gameDesignFence = tmp

local tmp = gameScreen:CreateSprite("07_bunker")
tmp:SetZpos(-940)
tmp:SetImage(tex07bunker)
tmp.position = Vector2D((screenSize.x - tmp:GetSize().x) / 2, screenSize.y - tmp:GetSize().y - 105)

local tmp = gameScreen:CreateSprite("08_camera")
tmp:SetZpos(-920)
tmp:SetImage(tex08camera)
tmp:SetCenter(Vector2D(28, 71))
tmp:SetPosition(Vector2D(406, 954))
gameDesignCamera = tmp

local tmp = gameScreen:CreateSprite("dead_background")
tmp:SetZpos(-500)
tmp:SetImage(texDeadBackground)
tmp:Show(false)

------------------------------------------------------------
-- Shoot flash
------------------------------------------------------------

model = ParticleModel(0.07, 0.1)
model:AddMutableParam("alpha", 0.4, 0)
model:AddFixedParam("blue", 1)
model:AddFixedParam("red", 1)
model:AddFixedParam("green", 1)

emitter = ParticleEmitter()
emitter:MakeDisk(Vector2D(0, 0), 0)

-- Particle
shootFlash = gameScreen:CreateParticleSource("shootFlashParticle", model, 6)
shootFlash:SetEmitter(emitter)
shootFlash:SetFlow(-1)
shootFlash:SetTank(0)
shootFlash:MakeRenderImages(texShootFlash, Vector2D(70, 70), true)
shootFlash.position = Vector2D(0, 0)

------------------------------------------------------------
-- Laser
------------------------------------------------------------
-- Small flame --
model = ParticleModel(1, 1)
model:AddRandomParam("alpha", 0.7, 1)
model:AddRandomParam("blue", 0, 0.5)
model:AddRandomParam("red", 0.7, 1)
model:AddRandomParam("green", 0, 0.3)
model:AddMutableRandomParam("size", 0.5, 1, 0.5, 1)
model:AddMutableRandomParam("angle", 0, 2 * 3.14, 0, 2 * 3.14)

emitter = ParticleEmitter()
emitter:MakeDisk(Vector2D(0, 0), 4)
emitter:SetAngles(0, 0)

-- Particle
particleLaser = gameScreen:CreateParticleSource("particle", model, 10000)
particleLaser:MakeRenderImages(texParticlePoint, Vector2D(60, 60), true)
particleLaser:SetEmitter(emitter)
particleLaser:SetFlow(0)
particleLaser:SetForce(800, 4000)
particleLaser:SetFriction(0)
particleLaser:SetGravity(Vector2D(0, 0))
particleLaser.position = Vector2D(0, 0)
particleLaser.zPos = 100

------------------------------------------------------------
-- Cash
------------------------------------------------------------
cashText = gameScreen:CreateText("cashTxt", "")
cashText:SetColor(Color(255, 255, 255))
cashText:SetSize(22)
cashText:SetZpos(10000)
cashText:SetPosition(Vector2D(47, 943))

function ChangeCash(deltaCash)
	cash = cash + deltaCash
	cashText:SetText(tostring(cash) .. " $")
end
ChangeCash(0)

------------------------------------------------------------
-- Level
------------------------------------------------------------
levelTextLabel = gameScreen:CreateText("lvlLabel", _("Level :"))
levelTextLabel:SetColor(Color(0, 0, 0))
levelTextLabel:SetSize(20)
levelTextLabelSize = levelTextLabel:GetTextSize()
levelTextLabel:SetPosition(Vector2D(screenSize.x - levelTextLabelSize.x - 20, screenSize.y - 120))
levelTextLabel:SetZpos(10000)

levelText = gameScreen:CreateText("lvlTxt", "")
levelText:SetColor(Color(255, 255, 255))
levelText:SetSize(22)
levelText:SetZpos(10000)

function RefreshLevel()
	local newLevel = math.floor(math.sqrt(score/8)+1)

	if newLevel ~= level or levelText:GetText() == "" then
		level = newLevel
	
		-- Cash + (level up)
		if levelText:GetText() ~= "" then
			ChangeCash(math.min(level * 1000, 20000)) -- Plafoné à 20 000
			
			-- Logo
			CreateSpriteFadeOutAnim(gameScreen, screenSize / 2 - texLevelUp:GetSize() / 2, texLevelUp, 2)
			
			-- Snd
			gameScreen:PlaySound(sndLevelUp)
			
			-- 20% de vie min lors des changements
			if life < maxLife * 0.2 then
				ChangeLife(maxLife * 0.2 - life)
			end
		end
	
		-- Level
		levelText:SetText(tostring(level))
		local txtSize = levelText:GetTextSize()
		levelText:SetPosition(Vector2D(1240 - txtSize.x, 943))
	end
end
RefreshLevel()


------------------------------------------------------------
-- Progress
------------------------------------------------------------
-- Life
lifeBar = stockSlice:CloneToGroup(gameScreen)
lifeBar:SetPosition(Vector2D(640, 954.7))

-- Bullet
bulletAmmoCircle = stockAmmoSlice:CloneToGroup(gameScreen)
--bulletAmmoCircle:SetPosition(Vector2D(406.5, 954.5))
bulletAmmoCircle:SetPosition(Vector2D(541, 954.7))
GroupSetAmmoProgress(bulletAmmoCircle, 1)

local tmpSprite = bulletAmmoCircle:CreateSprite("logo")
tmpSprite:SetImage(tex_bullet)
tmpSprite:SetPosition(tmpSprite:GetSize() / -2)

-- Laser
laserAmmoCircle = stockAmmoSlice:CloneToGroup(gameScreen)
laserAmmoCircle:SetPosition(Vector2D(738, 954.7))
GroupSetAmmoProgress(laserAmmoCircle, 1)

local tmpSprite = laserAmmoCircle:CreateSprite("logo")
tmpSprite:SetImage(laserLogoTex)
tmpSprite:SetPosition(tmpSprite:GetSize() / -2)

laserAmmoCircle:Show(canUseLaser)

-- FreezeWeapon
freezeWeaponAmmoCircle = stockAmmoSlice:CloneToGroup(gameScreen)
freezeWeaponAmmoCircle:SetPosition(Vector2D(807, 954.7))
GroupSetAmmoProgress(freezeWeaponAmmoCircle, 1)

local tmpSprite = freezeWeaponAmmoCircle:CreateSprite("logo")
tmpSprite:SetImage(freezeWeaponLogoTex)
tmpSprite:SetPosition(tmpSprite:GetSize() / -2)

freezeWeaponAmmoCircle:Show(canUseFreezeWeapon)

------------------------------------------------------------
-- Low life effect
------------------------------------------------------------
lowLifeEffect = gameScreen:CreatePostFX("lowLifeEffect")
lowLifeEffect:LoadFromFile("sfx/damage.sfx")
lowLifeEffect:SetZpos(10)
lowLifeEffect:SetParameter("effectDist", 0.2)


lowLifeEffectEvent = gameScreen:RegisterChronometer(	function(ratio, event)
															if ratio == 1 then
																event:Restart()
															end

															ratio = ratio * 2
															if ratio > 1 then
																ratio = 1.0 - (ratio - 1.0)
															end
															
															ratio = ConvertRatioCosAccelerationDeceleration(ratio)
															
															local skipedRatio = gameScreen:GetGlobalPosition().x / (screenSize.x + 2 * gameScreen:GetGlobalPosition().x)
															lowLifeEffect:SetParameter("effectDist", (0.1 + 0.2 * ratio) * (1 - skipedRatio * 2))
															lowLifeEffect:SetParameter("skipedRatio", skipedRatio)
														end, 3, true)

function lowLifeEffectEnable(isEnabled)
	if isEnabled then
		if not lowLifeEffectEvent:IsRunning() then
			lowLifeEffectEvent:Restart()
		end
	else
		lowLifeEffectEvent:Stop()
		lowLifeEffect:SetParameter("effectDist", 0)
	end
end

lowLifeEffectEnable(false)

------------------------------------------------------------
-- Life
------------------------------------------------------------
lifeText = gameScreen:CreateText("lifeTxt", "")
lifeText:SetColor(Color(0, 0, 0))
lifeText:SetZpos(10)
function RefreshLifeText()
	-- Text
	lifeText:SetText(tostring(math.floor(life)))
	txtSize = lifeText:GetTextSize()
	lifeText.position = lifeBar.position - txtSize / 2	
	
	-- Progress
	GroupSetProgress(lifeBar, life / maxLife)
	
	-- Health warning
	lowLifeEffectEnable(life < maxLife * 0.2) -- < 20% maxLife
end

function ChangeLife(alphaLife)
	if life == 0 then -- Return
		return
	end

	-- Damage
	life = life + alphaLife
	
	if alphaLife < 0 then
		lifeRecoverySpeed = 1 -- Life recovery speed is low when hit
		shieldFluctuationChrono:Restart()
	end
	
	if life < 0 then			
		life = 0
		gameCreateDeathAnim()
		--ChangeScreen(menuScreen)
	end
	
	RefreshLifeText()
end

function improveMaxLife(deltaMaxLife)
	maxLife = maxLife + deltaMaxLife
	ChangeLife(deltaMaxLife)
end

RefreshLifeText()
------------------------------------------------------------
-- Design smoke
------------------------------------------------------------
-- Smoke --
model = ParticleModel(3, 5)
model:AddMutableParam("alpha", 0.5, 0)
model:AddMutableRandomParam("size", 0, 0.1, 0.7, 1)
model:AddRandomParam("mass", 0.5, 1)
model:AddRandomParam("angle", 0, 2*3.14)

emitter = ParticleEmitter()
emitter:MakeDisk(Vector2D(0, 0), 20)
emitter:SetAngles(3.14 / 2, 0)
emitter:SetDirection(Vector2D(0, -1))

-- Particle
particleObj = gameScreen:CreateParticleSource("particle", model, 55)
particleObj:SetEmitter(emitter)
particleObj:SetFlow(10)
particleObj:SetForce(5, 15)
particleObj:MakeRenderImages(texSmoke1, Vector2D(70, 70), false)
particleObj:SetGravity(Vector2D(10, -30))
particleObj:SetFriction(0.2)
particleObj.position = Vector2D(170, 790)

particleObj2 = particleObj:Clone()
particleObj2.position = Vector2D(1000, 700)

particleObj:ComputeParticles(5)
particleObj2:ComputeParticles(5)

------------------------------------------------------------
-- Shield
------------------------------------------------------------
shieldSprite = gameScreen:CreateSprite("shield")
shieldSprite:SetZpos(-930)
shieldSprite:SetImage(tex_shield)
shieldSprite.y = 829
shieldSprite.color = Color(255, 255, 255, shieldFluctuationMin)

-- Shield => Physic
tmp = gameScreen:CreatePhLine(phGroupShield, "phShield", Vector2D(0, 846), Vector2D(screenSize.x, 846))
tmp:SetEventFunction(gameOnShieldShock)


------------------------------------------------------------
-- Gun
------------------------------------------------------------
gunGroup = gameScreen:CreateGroup("gun")
gunGroup:SetPosition(Vector2D(screenSize.x / 2, screenSize.y - 165)) -- Attention : sprite center has moved !
gunGroup:SetZpos(-950)

local tmp = gunGroup:CreateSprite("gunSpr")
tmp:SetZpos(0)
tmp:SetImage(tex06aGun)
tmp:SetCenter(Vector2D(tmp:GetSize().x / 2, tmp:GetSize().y))
tmp:SetRotation(0) -- Attention : sprite center has moved !

local tmp = gunGroup:CreateSprite("gunBaseSpr")
tmp:SetZpos(100)
tmp:SetImage(tex06bGunBase)
tmp:SetCenter(tmp:GetSize() / 2)
tmp:SetRotation(0) -- Attention : sprite center has moved !

	-- Sight
local tmp = gunGroup:CreateSprite("sightGen")
tmp:SetZpos(200)
tmp:Show(false)
tmp:SetImage(texSightGen)
tmp:SetPosition(Vector2D(-43, -125))

local tmp = gunGroup:CreateSprite("sight")
tmp:SetZpos(150)
tmp:Show(false)
tmp:SetImage(texSight)
tmp:SetCenter(Vector2D(texSight:GetWidth() / 2, texSight:GetHeight()))
tmp:SetPosition(Vector2D(-26.5, -120))
